package meerkat.voting.gui;

import javafx.stage.Stage;
import meerkat.voting.gui.ballot_summary.BallotSummaryLoader;
import meerkat.voting.gui.cast_or_audit.CastOrAuditLoader;
import meerkat.voting.gui.select_candidate_by_picture.SelectCandidateByPictureLoader;
import meerkat.voting.gui.select_candidate_name.SelectCandidateNameLoader;
import meerkat.voting.gui.straight_channel_section.StraightChannelSectionLoader;
import meerkat.voting.gui.vote_have_been_cast.VoteHaveBeenCastLoader;
import meerkat.voting.gui.welcome_splash.WelcomeSplashLoader;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by Vladimir Eliezer Tokarev on 8/27/2016.
 * ChainBuilder builds all the two way nodes that are in current project (voting-booth-gui)
 *              and connects between all of them so they can keep their flow
 */
class ChainBuilder {

    /**
     * Creates all  the twoWayNodes and connects between them
     *
     * The flow of Voting booth gui is next
     *      Welcome Splash screen
     *                \/
     *      Channel selection screen
     *                \/
     *      Select Candidate Name screen
     *         \/            /\
     *      Select candidate by picture screen
     *         \/            /\
     *      Ballot summary no pictures
     *                \/
     *      Cast Or audit screen
     *                \/
     *      thank you for screen
     * In order to maintain this order next technique was implemented every "screen" will have the next screen
     * and the previous this way the flow between will be easy .
     * The Build method creates all the screens and connect between them
     */
    static Map<String, TwoWayNode> Build(Stage primaryStage, VotingBoothConfiguration configuration) throws IOException {
        WelcomeSplashLoader welcomeSplashLoader = new WelcomeSplashLoader(primaryStage, configuration);
        TwoWayNode welcomeSplashController = welcomeSplashLoader.GetWelcomeSplash();

        StraightChannelSectionLoader straightChannelSectionLoader = new StraightChannelSectionLoader(primaryStage, configuration);
        TwoWayNode straightChannelSectionController = straightChannelSectionLoader.GetStraightChannelSection();
        welcomeSplashController.SetNext(straightChannelSectionController);
        straightChannelSectionController.SetPrevious(welcomeSplashController);

        SelectCandidateNameLoader selectCandidateNameLoader = new SelectCandidateNameLoader(primaryStage, configuration);
        TwoWayNode selectCandidateNameController = selectCandidateNameLoader.GetSelectCandidateName();
        selectCandidateNameController.SetPrevious(straightChannelSectionController);
        straightChannelSectionController.SetNext(selectCandidateNameController);

        SelectCandidateByPictureLoader selectCandidateByPictureLoader =
                new SelectCandidateByPictureLoader(primaryStage,configuration);
        TwoWayNode selectCandidateByPictureController = selectCandidateByPictureLoader.GetSelectCandidateByPicture();
        selectCandidateByPictureController.SetPrevious(selectCandidateNameController);
        selectCandidateNameController.SetNext(selectCandidateByPictureController);

        BallotSummaryLoader ballotSummaryLoader = new BallotSummaryLoader(primaryStage, configuration);
        TwoWayNode ballotSummaryController = ballotSummaryLoader.GetBallotSummary();
        ballotSummaryController.SetPrevious(selectCandidateByPictureController);
        selectCandidateByPictureController.SetNext(ballotSummaryController);

        CastOrAuditLoader castOrAuditLoader = new CastOrAuditLoader(primaryStage, configuration);
        TwoWayNode castOrAuditController = castOrAuditLoader.GetCastOrAudit();
        ballotSummaryController.SetNext(castOrAuditController);

        VoteHaveBeenCastLoader voteHaveBeenCastLoader = new VoteHaveBeenCastLoader(primaryStage, configuration);
        TwoWayNode voteHaveBeenCastController = voteHaveBeenCastLoader.GetVoteHaveBeenCast();
        castOrAuditController.SetNext(voteHaveBeenCastController);

        /**
         * all other two way nodes initialized
         */

        primaryStage.setScene(welcomeSplashController.GetNode());

        Map<String, TwoWayNode> nameToTwoWayNodeMap = new HashMap<>();
        nameToTwoWayNodeMap.put("welcomeSplashController", welcomeSplashController);
        nameToTwoWayNodeMap.put("straightChannelSectionController", straightChannelSectionController);
        nameToTwoWayNodeMap.put("selectCandidateNameController", selectCandidateNameController);
        nameToTwoWayNodeMap.put("selectCandidateByPictureController", selectCandidateByPictureController);
        nameToTwoWayNodeMap.put("ballotSummaryController", ballotSummaryController);
        nameToTwoWayNodeMap.put("voteHaveBeenCastController", voteHaveBeenCastController);

        return nameToTwoWayNodeMap;
    }
}
